#pragma once
//////////////////////////////////////////////////////////////////////
// geometryPositionColor.h: interface for the Geometry class.
// -Kyoung Shin Park (2014 Spring)
//////////////////////////////////////////////////////////////////////

#include "vertexBufferObject.h"

namespace moglclass
{

class VertexBufferObject;
class GeometryPositionColor
{
public:
	GeometryPositionColor();
	virtual ~GeometryPositionColor() { std::cout << "~Geometry" << std::endl; }
	virtual void init() = NULL;
    virtual void draw(bool wireframe=false) = NULL;
	virtual bool update(float dt);

	const float* getColor() { return &color[0]; }
	glm::vec3 Color() { return color; }
	void setColor(glm::vec3 color_) { color = color_; init(); };
	void setPosition(glm::vec3 p_) { p = p_; init(); };

	// animation
	void setAnimation(glm::vec3 start_, glm::vec3 end_, float duration_) { start = start_; end = end_; duration = duration_; p = start; init(); }
	void activate() { active = true; }
	void deactivate() { active = false; }
	bool isAnimationDone() { return done; }
	bool isActive() { return active; }
	void reset() { 	active = false; done = false; }

protected:
	glm::vec3 p; // center position
	glm::vec3 color; // color

	// vao & vbo
	int numVertices;
	GLuint vao;
    VertexBufferObject vbo;
	bool isLoaded;

	// animation
	bool done;
	bool active;
	glm::vec3 start;
	glm::vec3 end;
	float duration;
};

class Triangle : public GeometryPositionColor
{
public:
    Triangle(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 v1=glm::vec3(-1, -1, 0), glm::vec3 v2=glm::vec3(1, -1, 0), glm::vec3 v3=glm::vec3(0, 1, 0));
	void set(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_);
	void init();
	void draw(bool wireframe=false);
	
private:
	//glm::vec3 p; // center position
	glm::vec3 v1;
	glm::vec3 v2;
	glm::vec3 v3;
};

class Quad : public GeometryPositionColor
{
public:
	Quad(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 n=glm::vec3(0, 0, 1), glm::vec3 u=glm::vec3(0, 1, 0), float width=1.0f, float height=1.0f);
	void set(glm::vec3 p_, glm::vec3 n_, glm::vec3 u_, float width_, float height_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // center position
	glm::vec3 n; // normal vector
	glm::vec3 u; // up vector
	float width;
	float height;
};

class Circle : public GeometryPositionColor
{
public:
	Circle(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, int slices=8);
	void set(glm::vec3 p_, float radius_, int slices_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // center position
	float radius;
	int slices;
};

class Sphere : public GeometryPositionColor
{
public:
	Sphere(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, int slices=16, int stacks=16);
	void set(glm::vec3 p_, float radius_, int slices_, int stacks_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // center position
	float radius;
	int slices;
	int stacks;
};

class Cylinder : public GeometryPositionColor
{
public:
	Cylinder(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, float height=1.0f, int slices=16);
	void set(glm::vec3 p_, float radius_, float height_, int slices_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // center position
	float radius;
	float height;
	int slices;
};

class Cube : public GeometryPositionColor
{
public:
	Cube(glm::vec3 p_=glm::vec3(0, 0, 0), float size=1.0f);
	void set(glm::vec3 p_, float size_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // center position
	float size;
};

class Torus : public GeometryPositionColor
{
public:
	Torus(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, float tubeRadius=0.25f, int rings=16, int sides=16);
	void set(glm::vec3 p_, float radius_, float tubeRadius_, int rings_, int sides_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // center position
	float radius;
	float tubeRadius;
	int rings;
	int sides;
};

class Parallelepiped : public GeometryPositionColor
{
public:
	Parallelepiped(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 u=glm::vec3(1, 0, 0), glm::vec3 v=glm::vec3(0, 0, 1), glm::vec3 w=glm::vec3(0, 1, 0));
	void set(glm::vec3 p_, glm::vec3 u_, glm::vec3 v_, glm::vec3 w_);
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // BL position
	glm::vec3 u;
	glm::vec3 v;
	glm::vec3 w;
};

class Sa : public GeometryPositionColor 
{
public:
	Sa(glm::vec3 p=glm::vec3(0, 0, 0));
	void init();
	void draw(bool wireframe=false);

private:
	//glm::vec3 p; // BL position
	Parallelepiped pipe[4];
};

class Tree : public GeometryPositionColor
{
public:
	Tree(glm::vec3 p = glm::vec3(0, 0, 0));
	void init();
	void draw(bool wireframe = false);

private:
	Cylinder cyl;
	Sphere sph1;
	Sphere sph2;
};

}