/* geometry primitives using MVP */
#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"
#include "geometryPositionColor.h"

using namespace moglclass;

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

std::vector<GeometryPositionColor*> geoList;

int g_geometrymode = 0;
bool g_wiremode = false;
bool g_animationmode = false;

// using Uniform variables to shader
float g_fovy = 45.0f;
float g_aspect = 1.0f;
float g_zNear = 1.0f;
float g_zFar = 100.f;

glm::vec3 g_eye(0, 0, 5.0f);
glm::vec3 g_at(0, 0, 0);
glm::vec3 g_up(0, 1, 0);

// Model View Projection matrix
glm::mat4 World(1.0f);
glm::mat4 View(1.0f);
glm::mat4 Projection(1.0f);
glm::mat4 MVP(1.0f);

glm::mat4 yRotation(1.0f);

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

// set color triangle
void setGeometryData()
{
	GeometryPositionColor * geo = NULL;

	geo = new Tree(glm::vec3(0, 0, 0.0f));
	geoList.push_back(geo);
	geo = new Circle(glm::vec3(0, 0, -0.7f), 2.5f, 32);
	geo->setColor(glm::vec3(0, 1, 0)); // green circle
	geoList.push_back(geo);
	geo = new Triangle(glm::vec3(-1.5f, 0, 0), glm::vec3(0.0f, 1.5f, 0.0f), glm::vec3(-1.5f, -1.5f, 0.0f), glm::vec3(1.5f, -1.5f, 0.0f));
	geo->setColor(glm::vec3(1, 0, 0)); // red triangle
	geoList.push_back(geo);
	geo = new Triangle(glm::vec3(1.5f, 0, 0), glm::vec3(0.0f, 1.5f, 0.0f), glm::vec3(-1.5f, -1.5f, 0.0f), glm::vec3(1.5f, -1.5f, 0.0f));
	geo->setColor(glm::vec3(0, 0, 1)); // blue triangle
	geoList.push_back(geo);
	geo = new Quad(glm::vec3(0, 0, 0), glm::vec3(0, 0, 1.0f), glm::vec3(0, 1.0f, 0), 4.5f, 3.5f);
	geo->setColor(glm::vec3(0, 0, 1)); // blue quad
	geoList.push_back(geo);
	geo = new Sphere(glm::vec3(0.0f, 1.5f, 0.5f), 1.2f, 16, 16);
	geo->setColor(glm::vec3(1, 0, 1)); // margenta sphere
	geoList.push_back(geo);
	geo = new Cube(glm::vec3(0.0f, 0.7f, 0.0f), 2.5f);
	geo->setColor(glm::vec3(0, 1, 1)); // cyan cube
	geoList.push_back(geo);
	geo = new Cylinder(glm::vec3(-1.0f, -0.3f, 0.0f), 0.75f, 2.5f, 16);
	geo->setColor(glm::vec3(1, 1, 0)); // yellow cylinder
	geoList.push_back(geo);
	geo = new Cylinder(glm::vec3(1.0f, -0.3f, 0.0f), 0.75f, 2.5f, 16);
	geo->setColor(glm::vec3(0.8f, 0, 0.5f)); // pink cylinder
	geoList.push_back(geo);
	geo = new Torus(glm::vec3(0.0f, -0.5f, 0.5f), 1.2f, 0.3f, 16, 16);
	geo->setColor(glm::vec3(1, 0.5f, 0)); // orange torus
	geoList.push_back(geo);
	geo = new Parallelepiped(glm::vec3(0, 0, 0), glm::vec3(0.5f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.5f), glm::vec3(1.5f, 2.0f, 0.0f));
	geo->setColor(glm::vec3(0.2f, 0.8f, 1.0f)); // parallelepiped
	geoList.push_back(geo);
}

void drawGeometry()
{
	if (g_geometrymode == (int)geoList.size()) {
		// draw all geometry primitives together
		for (vector<GeometryPositionColor*>::const_iterator cii = geoList.begin(); cii != geoList.end(); ++cii)
		{
			(*cii)->draw();
		}
	}
	else {
		// draw geometry one at a time
		geoList[g_geometrymode]->draw();
	}
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple3.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple3.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();

	setGeometryData();

	glEnable(GL_DEPTH_TEST);
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); // clear the window

		// set drawmode
	if (g_wiremode) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// Model-View-Projection matrix
	Projection = glm::perspective(g_fovy, g_aspect, g_zNear, g_zFar);
	View = glm::lookAt(g_eye, g_at, g_up);
	World = yRotation;
	MVP = Projection * View * World;
	spMain.useProgram();
	spMain.setUniform("gMVP", MVP);

	drawGeometry();

	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

	glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int startTime, elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;

	if (g_animationmode)
	{
		// translate & rotate around x/y axis
		static float angle = 0.0f;
		angle += 0.001f * deltaTime;

		// geometry translate & rotate
		glm::mat4 xRotation;
		xRotation = glm::rotate(glm::mat4(1.0f), angle, glm::vec3(1, 0, 0));
		yRotation = glm::rotate(xRotation, angle, glm::vec3(0, 1, 0));
	}

	// each geometry update
	//geoList[0]->update((float)deltaTime);

	elapsedTime = currentTime - startTime;
	prevTime = currentTime;
	glutPostRedisplay();
}

void keyboard( unsigned char key, int x, int y )
{
	switch ( key ) {
		case 27:
			exit( EXIT_SUCCESS );
			break;
		case 'w':
			g_wiremode = !g_wiremode;
			printf("g_wiremode=%d\n", g_wiremode);
			break;
		case 'g':
			g_geometrymode = (g_geometrymode + 1) % ((int)geoList.size() + 1);
			printf("g_geometrymode=%d\n", g_geometrymode);
			break;
		case 'a':
			g_animationmode = !g_animationmode;
			printf("g_animationmode=%d\n", g_animationmode);
			break;
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y)
{
	switch (key) {
	case GLUT_KEY_UP:
		g_eye.z += 2;
		break;
	case GLUT_KEY_DOWN:
		g_eye.z -= 2;
		break;
	case GLUT_KEY_LEFT:
		g_at.x -= 1;
		g_eye.x -= 1;
		break;
	case GLUT_KEY_RIGHT:
		g_at.x += 1;
		g_eye.x += 1;
		break;
	}
	glutPostRedisplay();
}

void mouse(int button, int state, int mx, int my)
{
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN)
		exit(0);
}

void motion(int mx, int my)
{
	int w = glutGet(GLUT_WINDOW_WIDTH);
	int h = glutGet(GLUT_WINDOW_HEIGHT);
	float x = (float) (mx - w*0.5) / w;
	float y = (float) (h*0.5 - my) / h;

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	g_aspect = (float) (w/h);
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glutPostRedisplay();
}

void cleanup()
{
	std::vector<GeometryPositionColor*>::iterator iter = geoList.begin();
	while (iter != geoList.end()) {
		iter = geoList.erase(iter);
	}

	spMain.deleteProgram();
}

int main( int argc, char **argv )
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
	glutInitWindowSize( 600, 600 );
	glutCreateWindow( "3D Geometry Primitives" );

	glewInit();
	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutSpecialFunc( specialkey );
	glutIgnoreKeyRepeat(1);
	glutReshapeFunc( reshape );
	glutIdleFunc( update );
	glutMainLoop();

	cleanup();
	return 0;
}